// SamplDlg.cpp : implementation file
//

#include "stdafx.h"
#include "resource.h" // replace with theApp header if needed
#include "SamplDlg.h"
#include "opentsrv.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CSampleDlg dialog

CSampleDlg::CSampleDlg(CWnd* pParent /*=NULL*/)
	: CDialog(CSampleDlg::IDD, pParent)
{
	//{{AFX_DATA_INIT(CSampleDlg)
	m_DeviceID = _T("");
	m_ToDeviceID = _T("");
	//}}AFX_DATA_INIT
	// Note that LoadIcon does not require a subsequent DestroyIcon in Win32
	m_hIcon = AfxGetApp()->LoadIcon(IDR_MAINFRAME);
	m_TsapiWndPtr = 0;
}

CSampleDlg::~CSampleDlg()
{
	// delete the hidden TSAPI window
	if(m_TsapiWndPtr)
	{
		delete m_TsapiWndPtr;
	}
}

// For non-MFC folks, this is a helper routine for passing data back-and-forth
//  from this class to the dialog.
void CSampleDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CSampleDlg)
	DDX_Control(pDX, IDC_ANSWER, m_AnswerControl);
	DDX_Control(pDX, IDC_HOLD, m_HoldControl);
	DDX_Control(pDX, IDC_HANGUP, m_HangUpControl);
	DDX_Control(pDX, IDC_CALLLIST, m_CallListControl);
	DDX_Control(pDX, IDC_CALL, m_CallControl);
	DDX_Text(pDX, IDC_DEVICEID, m_DeviceID);
	DDX_Text(pDX, IDC_TODEVICEID, m_ToDeviceID);
	DDV_MaxChars(pDX, m_ToDeviceID, 64);
	//}}AFX_DATA_MAP
}

// For non-MFC folks, this map replaces the large "switch" statement found in
//  straight C Windows programs. The ON_MESSAGE() lines were added manually.
//  Read them as "when a message of type {first argument} is received, call
//   the method {second argument}"
BEGIN_MESSAGE_MAP(CSampleDlg, CDialog)
	ON_MESSAGE(WM_TSAPIFAILURE, OnTsapiFailure)
	ON_MESSAGE(WM_TSAPIACSUNSOLICITED, OnTsapiAcsUnsolicited)
	ON_MESSAGE(WM_TSAPICSTACONFIRMATION, OnTsapiCstaConfirmation)
	ON_MESSAGE(WM_TSAPICSTAUNSOLICITED, OnTsapiCstaUnsolicited)
	//{{AFX_MSG_MAP(CSampleDlg)
	ON_BN_CLICKED(IDC_CALL, OnCall)
	ON_BN_CLICKED(IDC_HANGUP, OnHangup)
	ON_EN_CHANGE(IDC_TODEVICEID, OnChangeTodeviceid)
	ON_WM_TIMER()
	ON_NOTIFY(NM_CLICK, IDC_CALLLIST, OnClickCalllist)
	ON_BN_CLICKED(IDC_HOLD, OnHold)
	ON_BN_CLICKED(IDC_ANSWER, OnAnswer)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CSampleDlg message handlers

BOOL CSampleDlg::OnInitDialog()
{
	CDialog::OnInitDialog();

	// Set the icon for this dialog.  The framework does this automatically
	//  when the application's main window is not a dialog
	SetIcon(m_hIcon, TRUE);			// Set big icon
	SetIcon(m_hIcon, FALSE);		// Set small icon

	// init the call list control
	m_CallListControl.InsertColumn(0, "Device ID", LVCFMT_LEFT, 90, 0);
	m_CallListControl.InsertColumn(LOCAL_CONNECTION_STATE, "Local Connection State",
									LVCFMT_LEFT, 150, LOCAL_CONNECTION_STATE);
	m_CallListControl.InsertColumn(REMOTE_CONNECTION_STATE, "Remote Connection State",
									LVCFMT_LEFT, 150, REMOTE_CONNECTION_STATE);

	// setting a timer allows the dialog to be displayed before initialization is
	//  completed - the init will be completed when the timer fires.
	if(!SetTimer(1, 1, NULL))
	{
		// if the timer can't be set, just call the OnTimer() method manually so
		//  initialization can complete.
		OnTimer(1);
	}	

	return TRUE;  // return TRUE  unless you set the focus to a control
}

void CSampleDlg::OnTimer(UINT nIDEvent) 
{
	// finish initialization...

	// ok to kill the timer even if it wasn't active (i.e. in the failure case from
	//  OnInitDialog().
	KillTimer(nIDEvent);

	// the class that calls COpenTsrv's DoModal() method is responsible
	//  for the CTsapiWnd object that will be created. It will be this
	//  classes responsibility to handle CTsapiWnd memory clean-up (delete
	//  the CTsapiWnd object) and also this classes responsibility to decide
	//  what to do on failures of the TSAPI stream.
	COpenTsrv openDlg;
	openDlg.m_TsapiController = (CWnd*)this;
	if(openDlg.DoModal() != IDOK)
	{
		PostMessage(WM_COMMAND, IDCANCEL, 0);
		return;
	}

	// retrieve the specified device to be monitored and the hidden TSAPI window
	m_DeviceID = openDlg.m_DeviceID;
	m_TsapiWndPtr = openDlg.m_TsapiWndPtr;

	// retrieve a pointer to the device record in the hidden TSAPI window
	DeviceID_t deviceID;
	lstrcpy(deviceID, m_DeviceID);
	CTsapiDevice* pTsapi = m_TsapiWndPtr->GetDeviceRecord(deviceID);
	if(!pTsapi)
	{
		AfxMessageBox("device record not found");
		PostMessage(WM_COMMAND, IDCANCEL, 0);
		return;
	}

	// Notify the TSAPI window that all events for this device should be forwarded
	//	to this dialog
	// Note that this is different than the m_TsapiController - which handles generic
	//  memory clean-up and TSAPI stream failures. SetWndPtr() is device-specific. If
	//  the app handled more than 1 device, they could have the same or different
	//  windows handling their events.
	pTsapi->SetWndPtr(this);

	UpdateData(FALSE);
	
	CDialog::OnTimer(nIDEvent);
}

// this method is called when the Call button is pressed
void CSampleDlg::OnCall() 
{
	// retrieve the data out of the dialog
	UpdateData(TRUE);

	// retrieve the device record for the monitored device
	DeviceID_t deviceID;
	lstrcpy(deviceID, m_DeviceID);
	CTsapiDevice* pTsapi = m_TsapiWndPtr->GetDeviceRecord(deviceID);
	if(!pTsapi)
	{
		AfxMessageBox("device record not found");
		PostMessage(WM_COMMAND, IDCANCEL, 0);
		return;
	}

	// make a call from monitored device to the "to device"
	// use the device record pointer as the invoke ID
	RetCode_t rc = cstaMakeCall(m_TsapiWndPtr->m_AcsHandle, (InvokeID_t)pTsapi, (DeviceID_t*)&deviceID,
									(DeviceID_t*)(LPCTSTR)m_ToDeviceID, NULL);
	if(rc < 0)
	{
		AfxMessageBox("make call failed");
		return;
	}

 	return;
}

// this method is called when the Hang Up button is pressed
void CSampleDlg::OnHangup() 
{
	// retrieve the data out of the dialog
	UpdateData(TRUE);

	// retrieve the device record for the monitored device
	DeviceID_t deviceID;
	lstrcpy(deviceID, m_DeviceID);
	CTsapiDevice* pTsapi = m_TsapiWndPtr->GetDeviceRecord(deviceID);
	if(!pTsapi)
	{
		AfxMessageBox("device record not found");
		PostMessage(WM_COMMAND, IDCANCEL, 0);
		return;
	}

	// get selected list item
	LV_ITEM lvItem;
	lvItem.iItem = m_CallListControl.GetNextItem(-1, LVNI_ALL | LVNI_SELECTED);
	lvItem.iSubItem = 0;
	lvItem.mask = LVIF_PARAM;
	m_CallListControl.GetItem(&lvItem);

	// retrieve the connection ID for this call that is associated with
	//  the monitored device - this is the "local" connection
	ConnectionID_t connID;
	if(!pTsapi->GetLocalConnection(lvItem.lParam, connID))
	{
		AfxMessageBox("no active call found");
		return;
	}

	// clear the connection
	// use the device record pointer as the invoke ID
	RetCode_t rc = cstaClearConnection(m_TsapiWndPtr->m_AcsHandle, (InvokeID_t)pTsapi,
											&connID, NULL);
	if(rc < 0)
	{
		AfxMessageBox("clear connection failed");
		return;
	}

	// disable hang-up and hold buttons
 	m_HangUpControl.EnableWindow(FALSE);
 	m_HoldControl.EnableWindow(FALSE);

	return;
}

// this method is called when the Answer button is pressed
void CSampleDlg::OnAnswer() 
{
	// retrieve the data out of the dialog
	UpdateData(TRUE);

	// retrieve the device record for the monitored device
	DeviceID_t deviceID;
	lstrcpy(deviceID, m_DeviceID);
	CTsapiDevice* pTsapi = m_TsapiWndPtr->GetDeviceRecord(deviceID);
	if(!pTsapi)
	{
		AfxMessageBox("device record not found");
		PostMessage(WM_COMMAND, IDCANCEL, 0);
		return;
	}

	// get selected list item
	LV_ITEM lvItem;
	lvItem.iItem = m_CallListControl.GetNextItem(-1, LVNI_ALL | LVNI_SELECTED);
	lvItem.iSubItem = 0;
	lvItem.mask = LVIF_PARAM;
	m_CallListControl.GetItem(&lvItem);

	// retrieve the connection ID for this call that is associated with
	//  the monitored device - this is the "local" connection
	ConnectionID_t connID;
	if(!pTsapi->GetLocalConnection(lvItem.lParam, connID))
	{
		AfxMessageBox("no active call found");
		return;
	}

	// answer the call
	// use the device record pointer as the invoke ID
	RetCode_t rc = cstaAnswerCall(m_TsapiWndPtr->m_AcsHandle, (InvokeID_t)pTsapi,
											&connID, NULL);
	if(rc < 0)
	{
		AfxMessageBox("answer call failed");
		return;
	}

	// disable the answer button
 	m_AnswerControl.EnableWindow(FALSE);

	return;
}

// this method is called when the Hold button is pressed
void CSampleDlg::OnHold() 
{
	// retrieve the data out of the dialog
	UpdateData(TRUE);

	// retrieve the device record for the monitored device
	DeviceID_t deviceID;
	lstrcpy(deviceID, m_DeviceID);
	CTsapiDevice* pTsapi = m_TsapiWndPtr->GetDeviceRecord(deviceID);
	if(!pTsapi)
	{
		AfxMessageBox("device record not found");
		PostMessage(WM_COMMAND, IDCANCEL, 0);
		return;
	}

	// get selected list item
	LV_ITEM lvItem;
	lvItem.iItem = m_CallListControl.GetNextItem(-1, LVNI_ALL | LVNI_SELECTED);
	lvItem.iSubItem = 0;
	lvItem.mask = LVIF_PARAM;
	m_CallListControl.GetItem(&lvItem);

	// retrieve the connection ID for this call that is associated with
	//  the monitored device - this is the "local" connection
	ConnectionID_t connID;
	if(!pTsapi->GetLocalConnection(lvItem.lParam, connID))
	{
		AfxMessageBox("no active call found");
		return;
	}

	// get the local connection state out of selected list control item
	CString connState = m_CallListControl.GetItemText(lvItem.iItem, LOCAL_CONNECTION_STATE);

	// If the local connection state is Held, then retrieve the call that
	//  is being held
	// else, put the call on hold
	// Note: the text of the hold button could also have been looked at
	//  to decide whether to hold or retrieve the call (if text == H&old,
	//  then hold the call, else retrieve the call).

	if(connState == _T("Held"))
	{
		// retrieve held call
		// use the device record pointer as the invoke ID
		RetCode_t rc = cstaRetrieveCall(m_TsapiWndPtr->m_AcsHandle, (InvokeID_t)pTsapi,
												&connID, NULL);
		if(rc < 0)
		{
			AfxMessageBox("retrieve call failed");
			return;
		}

		// now that the call has been retrieved, change button text
		m_HoldControl.SetWindowText(_T("H&old"));
	}
	else
	{
		// hold call - just set reservation flag to FALSE
		RetCode_t rc = cstaHoldCall(m_TsapiWndPtr->m_AcsHandle, (InvokeID_t)pTsapi,
												&connID, FALSE, NULL);
		if(rc < 0)
		{
			AfxMessageBox("hold call failed");
			return;
		}

		// now that the call has been held, change button text
		m_HoldControl.SetWindowText(_T("Unh&old"));
	}
	
	return;
}

// this method enables/disables buttons (and input fields) based on current status
void CSampleDlg::DoButtonUpdates() 
{
	// get selected list item
	int index = m_CallListControl.GetNextItem(-1, LVNI_ALL | LVNI_SELECTED);
	if(index < 0)
	{
		// if no item is selected, nothing can be done
		m_HoldControl.EnableWindow(FALSE);
		m_AnswerControl.EnableWindow(FALSE);
		m_HangUpControl.EnableWindow(FALSE);
		return;
	}

	// get the local connection state out of selected list control item
	CString connState = m_CallListControl.GetItemText(index, LOCAL_CONNECTION_STATE);

	// enable hold button - for holding or unholding
	m_HoldControl.EnableWindow(TRUE);
	// set hold text based on selected item's state
	m_HoldControl.SetWindowText((connState == _T("Held")) ? _T("Unh&old") : _T("H&old"));
	// enable answer and hang-up buttons based on selected item's state
	m_AnswerControl.EnableWindow(connState == _T("Alerting"));
	m_HangUpControl.EnableWindow(connState == _T("Connected"));	
}

// this method is called when an item in the call list is selected
void CSampleDlg::OnClickCalllist(NMHDR* pNMHDR, LRESULT* pResult) 
{
	// update the buttons based on state of the selected item
	DoButtonUpdates();
	*pResult = 0;
}

// this method is called on every update to the "Make Call To" field.
void CSampleDlg::OnChangeTodeviceid() 
{
	// retrieve the data out of the dialog
	UpdateData(TRUE);
	// enable the Call button if the "Make Call To" field is not empty
	m_CallControl.EnableWindow(!m_ToDeviceID.IsEmpty());	
}

// this method is called when the CSTA DLL returns failure on an attempt
//  to retrieve a TSAPI message.
// the failure code is in wParam. It is a RetCode_t and is one of the
//  failure codes defined in acs.h.
LRESULT CSampleDlg::OnTsapiFailure(WPARAM wParam, LPARAM lParam)
{
	char tempStr[10];
	sprintf(tempStr, "TSAPI Message Retrieval Failure: %d", wParam);
	AfxMessageBox(tempStr);
	PostMessage(WM_COMMAND, IDCANCEL, 0);
	return 0;
}

// this method is called when an ACS Unsolicited event is received
LRESULT CSampleDlg::OnTsapiAcsUnsolicited(WPARAM wParam, LPARAM lParam)
{
	char tempStr[10];
	sprintf(tempStr, "TSAPI Stream Failure: %d",
			m_TsapiWndPtr->m_EventBufPtr->event.acsUnsolicited.u.failureEvent.error);
	AfxMessageBox(tempStr);
	PostMessage(WM_COMMAND, IDCANCEL, 0);
	return 0;
}

// this method is called when a CSTA Confirmation event is received
LRESULT CSampleDlg::OnTsapiCstaConfirmation(WPARAM wParam, LPARAM lParam)
{
	switch(m_TsapiWndPtr->m_EventBufPtr->eventHeader.eventType)
	{
	 case CSTA_MAKE_CALL_CONF:
	 case CSTA_CLEAR_CONNECTION_CONF:
	 case CSTA_ANSWER_CALL_CONF:
	 case CSTA_HOLD_CALL_CONF:
	 case CSTA_RETRIEVE_CALL_CONF:
		// let the unsolicited events take care of any updates
		// Generally, nothing should be done with positive confirmation
		//  events - the unsolicited events should drive the application's
		//  behaviour.
		break;
	 case CSTA_UNIVERSAL_FAILURE_CONF:
		// something more elaborate should probably be done when
		//  a request fails!
		AfxMessageBox("requested operation failed");
		break; 
	 default:
		AfxMessageBox("unexpected response to requested operation");
		break; 
	}

	return 0;
}

// this method is called when a CSTA Unsolicited event is received
LRESULT CSampleDlg::OnTsapiCstaUnsolicited(WPARAM wParam, LPARAM lParam)
{
	// Get the device record pointer out of the lParam
	// Get the unsolicited event pointer out of the event buffer (for
	//  code efficiency and typing efficiency)
	CTsapiDevice* pTsapi = (CTsapiDevice*)lParam;
	CSTAUnsolicitedEvent* pUns = &(m_TsapiWndPtr->m_EventBufPtr->event.cstaUnsolicited);

	switch(m_TsapiWndPtr->m_EventBufPtr->eventHeader.eventType)
	{
	 // this is not a call-control event. just end the app if the monitor
	 //  of the device was terminated for any reason.
	 case CSTA_MONITOR_ENDED:
		AfxMessageBox("monitor failure");
		PostMessage(WM_COMMAND, IDCANCEL, 0);
		return 0;

	 // these are call-control events...
	 case CSTA_SERVICE_INITIATED: // null -> initiated (LOCAL only)
	 {
		ConnectionID_t* pConnID = &(pUns->u.serviceInitiated.initiatedConnection);

		// this is a new call - track it
		long callHandle = pTsapi->CreateCallFromLocalConnection(*pConnID);
		if(callHandle == 0)
		{
			AfxMessageBox("no memory to add new call");
			return 0;
		}

		LV_ITEM lvItem;
		lvItem.iItem = 0;
		lvItem.iSubItem = 0;
		lvItem.mask = LVIF_PARAM;
		lvItem.lParam = callHandle;

		// insert the call into the list control
		int listIndex = m_CallListControl.InsertItem(&lvItem);
		if(listIndex < 0)
		{
			pTsapi->RemoveCallViaConnection(*pConnID);
			AfxMessageBox("no memory to add new call");
			return 0;
		}

		// this event is always for the local connection
	 	m_CallListControl.SetItemText(listIndex, LOCAL_CONNECTION_STATE, _T("Initiated"));
		break;
	 }
	 case CSTA_ORIGINATED: // null/initiated -> connected (LOCAL only)
	 {
		ConnectionID_t* pConnID = &(pUns->u.originated.originatedConnection);

		// Attempt to find the call associated with the connection ID
		//  if a SERVICE INITIATED event was received, this call will
		//   be found.
		//  if no SERVICE INITIATED event was received, then this event is the
		//   first event received.
		long callHandle = pTsapi->GetCallHandle(*pConnID, TRUE);

		int listIndex;

		if(callHandle == 0)
		{
			// assume this is a new call - track it
			callHandle = pTsapi->CreateCallFromLocalConnection(*pConnID);
			if(callHandle == 0)
			{
				AfxMessageBox("no memory to add new call");
				return 0;
			}

			LV_ITEM lvItem;
			lvItem.iItem = 0;
			lvItem.iSubItem = 0;
			lvItem.mask = LVIF_PARAM;
			lvItem.lParam = callHandle;

			// insert the call into the list control
			listIndex = m_CallListControl.InsertItem(&lvItem);
			if(listIndex < 0)
			{
				pTsapi->RemoveCallViaConnection(*pConnID);
				AfxMessageBox("no memory to add new call");
				return 0;
			}
		}
		else
		{
			LV_FINDINFO findInfo;
			findInfo.flags = LVFI_PARAM;
			findInfo.lParam = callHandle;
			listIndex = m_CallListControl.FindItem(&findInfo, -1);
		}

		// this event is always for the local connection
		m_CallListControl.SetItemText(listIndex, LOCAL_CONNECTION_STATE, _T("Connected"));

		CString calledDevice;

		// get the number dialed out of calledDevice
		// For calls originated via the phone, this is the first chance to
		//  place the information into the dialog.
		// Modifying the "To Device" here and in other events illustrates
		//  how the PBX and/or PBX Driver what has been dialled. Apps need
		//  to be aware that this can happen.
		if(pUns->u.originated.calledDevice.deviceIDStatus == ID_PROVIDED)
		{
			calledDevice = pUns->u.originated.calledDevice.deviceID;
		}
		else
		{
			calledDevice = _T("Not provided");
		}

		m_CallListControl.SetItemText(listIndex, 0, calledDevice);
		break;
	 }
	 case CSTA_DELIVERED: // null -> alerting
	 {
		ConnectionID_t* pConnID = &(pUns->u.delivered.connection);
 		SubjectDeviceID_t* pSubjectDevice = &(pUns->u.delivered.alertingDevice);
 		CallingDeviceID_t* pCallingDevice = &(pUns->u.delivered.callingDevice);
 		CalledDeviceID_t* pCalledDevice = &(pUns->u.delivered.calledDevice);

		BOOL subjectProvided = (pSubjectDevice->deviceIDStatus == ID_PROVIDED);
		BOOL callingProvided = (pCallingDevice->deviceIDStatus == ID_PROVIDED);
		BOOL calledProvided = (pCalledDevice->deviceIDStatus == ID_PROVIDED);

		BOOL outgoing;
		if(subjectProvided)
		{
			outgoing = !(m_DeviceID == pSubjectDevice->deviceID);
		}
		else if(callingProvided)
		{
			outgoing = (m_DeviceID == pCallingDevice->deviceID);
		}
		else if(calledProvided)
		{
			outgoing = !(m_DeviceID == pCalledDevice->deviceID);
		}
		else
		{
			// nothing we can do if no IDs are provided!
			return 0;
		}

		if(outgoing)
		{
			// don't bother storing remote connection while it is still alerting...

			// Find the call handle based on call ID in given connection ID
			// Unfortunately, we must crack open the ConnID here (the FALSE indicates
			//  an inexact match - just match on callID). Looking at the callID is
			//  not guaranteed to work on 100% of all PBX drivers - some implementations
			//  do not guarantee unique call IDs, only unique connection IDs.
			//  There are 2 alternatives to this, neither one of which is any better
			//  than this (and may even be worse).
			//  1. Look at the calledDevice. The app should know the number that was called,
			//		and could store that number with the local connection ID. When execution
			//		reaches here, search for the called device and return the callHandle. This
			//		has 2 problems: a. calledDevice is not guaranteed to be supplied from all PBX
			//		drivers; b. if 2 calls to the same device are in progress simultaneously, which
			//		callHandle is for which device?
			//  2. Do a cstaSnapshotCall() on the connection ID passed in here. That will give you
			//		a picture of what is happening with that connID - including the connID in the
			//		call that is associated with the monitored device. The snapshot can be used to
			//		match that local connID to what is stored in the app. This delays handling of
			//		the event with asynchronous queries to the server - in the meantime other things
			//		are happening to the call!
			long callHandle = pTsapi->GetCallHandle(*pConnID, FALSE);
			if(callHandle == 0)
			{
				return 0;
			}

			// need to get listIndex for where this message should go
 			LV_FINDINFO findInfo;
			findInfo.flags = LVFI_PARAM;
			findInfo.lParam = callHandle;
			int listIndex = m_CallListControl.FindItem(&findInfo, -1);

			m_CallListControl.SetItemText(listIndex, REMOTE_CONNECTION_STATE, _T("Alerting"));

			CString calledDevice;

			// get the called device out of subjectDevice or calledDevice
			if(subjectProvided)
			{
				calledDevice = pSubjectDevice->deviceID;
			}
			else if(calledProvided)
			{
				calledDevice = pCalledDevice->deviceID;
			}
			else
			{
				calledDevice = _T("Not provided");
			}

		 	m_CallListControl.SetItemText(listIndex, 0, calledDevice);
		}
		else
		{
			// this is a new call (for this device) - track it
			long callHandle = pTsapi->CreateCallFromLocalConnection(*pConnID);
			if(callHandle == 0)
			{
				AfxMessageBox("no memory to add new call");
				return 0;
			}

			LV_ITEM lvItem;
			lvItem.iItem = 0;
			lvItem.iSubItem = 0;
			lvItem.mask = LVIF_PARAM;
			lvItem.lParam = callHandle;

			// insert the call into the list control
			int listIndex = m_CallListControl.InsertItem(&lvItem);
			if(listIndex < 0)
			{
				pTsapi->RemoveCallViaConnection(*pConnID);
				AfxMessageBox("no memory to add new call");
				return 0;
			}

		 	m_CallListControl.SetItemText(listIndex, LOCAL_CONNECTION_STATE, _T("Alerting"));

			CString callingDevice;

			// get the caller out of callingDevice
			if(callingProvided)
			{
				callingDevice = pCallingDevice->deviceID;
			}
			else
			{
				callingDevice = _T("Not provided");
			}

		 	m_CallListControl.SetItemText(listIndex, 0, callingDevice);
			// update the remote status	manually to "connected" since the remote
			//  connection existed before this monitor knew about the call.
			m_CallListControl.SetItemText(listIndex, REMOTE_CONNECTION_STATE, _T("Connected"));
		}
		break;
	 }
	 case CSTA_NETWORK_REACHED: // null -> connected (REMOTE only)
	 {
		ConnectionID_t* pConnID = &(pUns->u.networkReached.connection);
		SubjectDeviceID_t* pSubjectDevice = &(pUns->u.networkReached.trunkUsed);
		CalledDeviceID_t* pCalledDevice = &(pUns->u.networkReached.calledDevice);

		// Find the call handle based on call ID in given connection ID
		// Unfortunately, we must crack open the ConnID here (the FALSE indicates
		//  an inexact match - just match on callID). Looking at the callID is
		//  not guaranteed to work on 100% of all PBX drivers - some implementations
		//  do not guarantee unique call IDs, only unique connection IDs.
		//  There are 2 alternatives to this, neither one of which is any better
		//  than this (and may even be worse).
		//  1. Look at the calledDevice. The app should know the number that was called,
		//		and could store that number with the local connection ID. When execution
		//		reaches here, search for the called device and return the callHandle. This
		//		has 2 problems: a. calledDevice is not guaranteed to be supplied from all PBX
		//		drivers; b. if 2 calls to the same device are in progress simultaneously, which
		//		callHandle is for which device?
		//  2. Do a cstaSnapshotCall() on the connection ID passed in here. That will give you
		//		a picture of what is happening with that connID - including the connID in the
		//		call that is associated with the monitored device. The snapshot can be used to
		//		match that local connID to what is stored in the app. This delays handling of
		//		the event with asynchronous queries to the server - in the meantime other things
		//		are happening to the call!
		long callHandle = pTsapi->GetCallHandle(*pConnID, FALSE);
		if(callHandle == 0)
		{
			return 0;
		}

		// now add the connection to the call - while this connection is in
		//  the call, all future events should not need to use GetCallHandle() with FALSE,
		//  they should all use TRUE.

		if(!pTsapi->AddConnectionToCall(callHandle, *pConnID))
		{
			AfxMessageBox("no memory to add new connection to call");
			return 0;
		}

		LV_FINDINFO findInfo;
		findInfo.flags = LVFI_PARAM;
		findInfo.lParam = callHandle;
		int listIndex = m_CallListControl.FindItem(&findInfo, -1);

		// This event is always for the remote connection
		m_CallListControl.SetItemText(listIndex, REMOTE_CONNECTION_STATE, _T("Connected"));

		CString trunkUsed;

		// get the trunk used out of subjectDevice or calledDevice
		if(pSubjectDevice->deviceIDStatus == ID_PROVIDED)
		{
			trunkUsed = pSubjectDevice->deviceID;
		}
		else if(pCalledDevice->deviceIDStatus == ID_PROVIDED)
		{
			trunkUsed = pCalledDevice->deviceID;
		}
		else
		{
			trunkUsed = _T("Not provided");
		}

		m_CallListControl.SetItemText(listIndex, 0, trunkUsed);
		break;
	 }
	 case CSTA_ESTABLISHED:	// alerting/connected -> connected
	 {
		ConnectionID_t* pConnID = &(pUns->u.established.establishedConnection);
 		SubjectDeviceID_t* pSubjectDevice = &(pUns->u.established.answeringDevice);
 		CallingDeviceID_t* pCallingDevice = &(pUns->u.established.callingDevice);
 		CalledDeviceID_t* pCalledDevice = &(pUns->u.established.calledDevice);

  		BOOL subjectProvided = (pSubjectDevice->deviceIDStatus == ID_PROVIDED);
		BOOL callingProvided = (pCallingDevice->deviceIDStatus == ID_PROVIDED);
		BOOL calledProvided = (pCalledDevice->deviceIDStatus == ID_PROVIDED);
		BOOL outgoing = FALSE;

		// Attempt to find the call associated with the connection ID
		long callHandle = pTsapi->GetCallHandle(*pConnID, TRUE);
		if(callHandle == 0)
		{
			// if the call is outgoing to an internal destination (no NETWORK REACHED event
			//  has been received) we will get here.

			// is it outgoing?
			if(subjectProvided)
			{
				outgoing = !(m_DeviceID == pSubjectDevice->deviceID);
			}
			else if(callingProvided)
			{
				outgoing = (m_DeviceID == pCallingDevice->deviceID);
			}
			else if(calledProvided)
			{
				outgoing = !(m_DeviceID == pCalledDevice->deviceID);
			}
			else
			{
				// nothing we can do if no IDs are provided!
				return 0;
			}

			// if it is NOT outgoing, ignore event - return here
			if(!outgoing)
			{
				return 0;
			}

			// Find the call handle based on call ID in given connection ID
			// Unfortunately, we must crack open the ConnID here (the FALSE indicates
			//  an inexact match - just match on callID). Looking at the callID is
			//  not guaranteed to work on 100% of all PBX drivers - some implementations
			//  do not guarantee unique call IDs, only unique connection IDs.
			//  There are 2 alternatives to this, neither one of which is any better
			//  than this (and may even be worse).
			//  1. Look at the calledDevice. The app should know the number that was called,
			//		and could store that number with the local connection ID. When execution
			//		reaches here, search for the called device and return the callHandle. This
			//		has 2 problems: a. calledDevice is not guaranteed to be supplied from all PBX
			//		drivers; b. if 2 calls to the same device are in progress simultaneously, which
			//		callHandle is for which device?
			//  2. Do a cstaSnapshotCall() on the connection ID passed in here. That will give you
			//		a picture of what is happening with that connID - including the connID in the
			//		call that is associated with the monitored device. The snapshot can be used to
			//		match that local connID to what is stored in the app. This delays handling of
			//		the event with asynchronous queries to the server - in the meantime other things
			//		are happening to the call!
			callHandle = pTsapi->GetCallHandle(*pConnID, FALSE);
			if(callHandle == 0)
			{
				return 0;
			}

			// now add the connection to the call - while this connection is in
			//  the call, all future events should not need to use GetCallHandle() with FALSE,
			//  they should all use TRUE.

			if(!pTsapi->AddConnectionToCall(callHandle, *pConnID))
			{
				AfxMessageBox("no memory to add new connection to call");
				return 0;
			}
		}

  		LV_FINDINFO findInfo;
		findInfo.flags = LVFI_PARAM;
		findInfo.lParam = callHandle;
		int listIndex = m_CallListControl.FindItem(&findInfo, -1);

		if(outgoing)
		{
			m_CallListControl.SetItemText(listIndex, REMOTE_CONNECTION_STATE, _T("Connected"));

			CString answeringDevice;

			// get the answering device out of subjectDevice or calledDevice
			if(subjectProvided)
			{
				answeringDevice = pSubjectDevice->deviceID;
			}
			else if(calledProvided)
			{
				answeringDevice = pCalledDevice->deviceID;
			}
			else
			{
				answeringDevice = _T("Not provided");
			}

		 	m_CallListControl.SetItemText(listIndex, 0, answeringDevice);
		}
		else
		{
		 	m_CallListControl.SetItemText(listIndex, LOCAL_CONNECTION_STATE, _T("Connected"));
		}
		break;
	 }
	 case CSTA_DIVERTED: // alerting -> null
	 {
		ConnectionID_t* pConnID = &(pUns->u.diverted.connection);

		// The conn ID is always known to the app here.
		// This event is for the local connection on an incoming call only
		// The call is being diverted away from the monitored device,
		//  so remove the call from the application
		long callHandle = pTsapi->RemoveCallViaConnection(*pConnID);
		if(callHandle == 0)
		{
			return 0;
		}

  		LV_FINDINFO findInfo;
		findInfo.flags = LVFI_PARAM;
		findInfo.lParam = callHandle;
		int listIndex = m_CallListControl.FindItem(&findInfo, -1);

		// delete the selected list control item
		m_CallListControl.DeleteItem(listIndex);
		break;
	 }
	 case CSTA_HELD: // connected -> held
	 {
		ConnectionID_t* pConnID = &(pUns->u.held.heldConnection);

		// Attempt to find the call associated with the connection ID
		long callHandle = pTsapi->GetCallHandle(*pConnID, TRUE);
		if(callHandle == 0)
		{
			return 0;
		}

		// need to get listIndex for where this message should go
 		LV_FINDINFO findInfo;
		findInfo.flags = LVFI_PARAM;
		findInfo.lParam = callHandle;
		int listIndex = m_CallListControl.FindItem(&findInfo, -1);

		// If the connection ID is a "local" connection id (i.e. it is associated with
		//  the monitored device), then update the local connection state.
		// else update the remote connection state.
		// For you folks who like fancy code, use the following instead of the if-else:
		// m_CallListControl.SetItemText(listIndex,
		//							(pTsapi->IsLocalConnection(callHandle, *pConnID)) ?
		//							LOCAL_CONNECTION_STATE : REMOTE_CONNECTION_STATE,
		//							_T("Held"));
		if(pTsapi->IsLocalConnection(callHandle, *pConnID))
		{
		 	m_CallListControl.SetItemText(listIndex, LOCAL_CONNECTION_STATE, _T("Held"));
		}
		else
		{
		 	m_CallListControl.SetItemText(listIndex, REMOTE_CONNECTION_STATE, _T("Held"));
		}
		break;
	 }
	 case CSTA_RETRIEVED: // held -> connected
	 {
		ConnectionID_t* pConnID = &(pUns->u.retrieved.retrievedConnection);

		// Attempt to find the call associated with the connection ID
		long callHandle = pTsapi->GetCallHandle(*pConnID, TRUE);
		if(callHandle == 0)
		{
			return 0;
		}

		// need to get listIndex for where this message should go
 		LV_FINDINFO findInfo;
		findInfo.flags = LVFI_PARAM;
		findInfo.lParam = callHandle;
		int listIndex = m_CallListControl.FindItem(&findInfo, -1);

		// If the connection ID is a "local" connection id (i.e. it is associated with
		//  the monitored device), then update the local connection state.
		// else update the remote connection state.
		if(pTsapi->IsLocalConnection(callHandle, *pConnID))
		{
		 	m_CallListControl.SetItemText(listIndex, LOCAL_CONNECTION_STATE, _T("Connected"));
		}
		else
		{
		 	m_CallListControl.SetItemText(listIndex, REMOTE_CONNECTION_STATE, _T("Connected"));
		}
		break;
	 }
	 case CSTA_CALL_CLEARED: // * -> null (for all connections in call)
	 {
		ConnectionID_t* pConnID = &(pUns->u.callCleared.clearedCall);

		// remove the call associated with the connection ID
		long callHandle = pTsapi->RemoveCallViaConnection(*pConnID);
		if(callHandle == 0)
		{
			//  If the call was not known, just return
			return 0;
		}

  		LV_FINDINFO findInfo;
		findInfo.flags = LVFI_PARAM;
		findInfo.lParam = callHandle;
		int listIndex = m_CallListControl.FindItem(&findInfo, -1);

		// delete the selected list control item
		m_CallListControl.DeleteItem(listIndex);
		break;
	 }
	 case CSTA_CONNECTION_CLEARED: // * -> null
	 {
		ConnectionID_t* pConnID = &(pUns->u.connectionCleared.droppedConnection);

		// remove the call associated with the connection ID
		long callHandle = pTsapi->RemoveCallViaConnection(*pConnID);
		if(callHandle == 0)
		{
			//  If the call was not known, just return
			return 0;
		}

		// need to get listIndex for where this message should go
 		LV_FINDINFO findInfo;
		findInfo.flags = LVFI_PARAM;
		findInfo.lParam = callHandle;
		int listIndex = m_CallListControl.FindItem(&findInfo, -1);

		// delete the selected list control item
		m_CallListControl.DeleteItem(listIndex);
		break;
	 }
	 default:
		// unhandled message - ignore it
		return 0;
	}

	// update the status of buttons and fields
	DoButtonUpdates();
	// update the contents of fields
	UpdateData(FALSE);
	return 0;
}

