// SamplDlg.cpp : implementation file
//

#include "stdafx.h"
#include "resource.h" // replace with theApp header if needed
#include "SamplDlg.h"
#include "opentsrv.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CSampleDlg dialog

CSampleDlg::CSampleDlg(CWnd* pParent /*=NULL*/)
	: CDialog(CSampleDlg::IDD, pParent)
{
	//{{AFX_DATA_INIT(CSampleDlg)
	m_DeviceID = _T("");
	m_ToDeviceID = _T("");
	m_LocalStatus = _T("Null");
	m_RemoteStatus = _T("Null");
	//}}AFX_DATA_INIT
	// Note that LoadIcon does not require a subsequent DestroyIcon in Win32
	m_hIcon = AfxGetApp()->LoadIcon(IDR_MAINFRAME);
	m_TsapiWndPtr = 0;
	m_CallHandle = 0;
}

CSampleDlg::~CSampleDlg()
{
	// delete the hidden TSAPI window
	if(m_TsapiWndPtr)
	{
		delete m_TsapiWndPtr;
	}
}

// For non-MFC folks, this is a helper routine for passing data back-and-forth
//  from this class to the dialog.
void CSampleDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CSampleDlg)
	DDX_Control(pDX, IDC_TODEVICEID, m_ToDeviceIDControl);
	DDX_Control(pDX, IDC_HANGUP, m_HangUpControl);
	DDX_Control(pDX, IDC_CALL, m_CallControl);
	DDX_Text(pDX, IDC_DEVICEID, m_DeviceID);
	DDX_Text(pDX, IDC_TODEVICEID, m_ToDeviceID);
	DDV_MaxChars(pDX, m_ToDeviceID, 64);
	DDX_Text(pDX, IDC_LOCALSTATUS, m_LocalStatus);
	DDX_Text(pDX, IDC_REMOTESTATUS, m_RemoteStatus);
	//}}AFX_DATA_MAP
}

// For non-MFC folks, this map replaces the large "switch" statement found in
//  straight C Windows programs. The ON_MESSAGE() lines were added manually.
//  Read them as "when a message of type {first argument} is received, call
//   the method {second argument}"
BEGIN_MESSAGE_MAP(CSampleDlg, CDialog)
	ON_MESSAGE(WM_TSAPIFAILURE, OnTsapiFailure)
	ON_MESSAGE(WM_TSAPIACSUNSOLICITED, OnTsapiAcsUnsolicited)
	ON_MESSAGE(WM_TSAPICSTACONFIRMATION, OnTsapiCstaConfirmation)
	ON_MESSAGE(WM_TSAPICSTAUNSOLICITED, OnTsapiCstaUnsolicited)
	//{{AFX_MSG_MAP(CSampleDlg)
	ON_BN_CLICKED(IDC_CALL, OnCall)
	ON_BN_CLICKED(IDC_HANGUP, OnHangup)
	ON_EN_CHANGE(IDC_TODEVICEID, OnChangeTodeviceid)
	ON_WM_TIMER()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CSampleDlg message handlers

BOOL CSampleDlg::OnInitDialog()
{
	CDialog::OnInitDialog();

	// Set the icon for this dialog.  The framework does this automatically
	//  when the application's main window is not a dialog
	SetIcon(m_hIcon, TRUE);			// Set big icon
	SetIcon(m_hIcon, FALSE);		// Set small icon

	// setting a timer allows the dialog to be displayed before initialization is
	//  completed - the init will be completed when the timer fires.
	if(!SetTimer(1, 1, NULL))
	{
		// if the timer can't be set, just call the OnTimer() method manually so
		//  initialization can complete.
		OnTimer(1);
	}	

	return TRUE;  // return TRUE  unless you set the focus to a control
}

void CSampleDlg::OnTimer(UINT nIDEvent) 
{
	// finish initialization...

	// ok to kill the timer even if it wasn't active (i.e. in the failure case from
	//  OnInitDialog().
	KillTimer(nIDEvent);

	// the class that calls COpenTsrv's DoModal() method is responsible
	//  for the CTsapiWnd object that will be created. It will be this
	//  classes responsibility to handle CTsapiWnd memory clean-up (delete
	//  the CTsapiWnd object) and also this classes responsibility to decide
	//  what to do on failures of the TSAPI stream.
	COpenTsrv openDlg;
	openDlg.m_TsapiController = (CWnd*)this;
	if(openDlg.DoModal() != IDOK)
	{
		PostMessage(WM_COMMAND, IDCANCEL, 0);
		return;
	}

	// retrieve the specified device to be monitored and the hidden TSAPI window
	m_DeviceID = openDlg.m_DeviceID;
	m_TsapiWndPtr = openDlg.m_TsapiWndPtr;

	// retrieve a pointer to the device record in the hidden TSAPI window
	DeviceID_t deviceID;
	lstrcpy(deviceID, m_DeviceID);
	CTsapiDevice* pTsapi = m_TsapiWndPtr->GetDeviceRecord(deviceID);
	if(!pTsapi)
	{
		AfxMessageBox("device record not found");
		PostMessage(WM_COMMAND, IDCANCEL, 0);
		return;
	}

	// Notify the TSAPI window that all events for this device should be forwarded
	//	to this dialog
	// Note that this is different than the m_TsapiController - which handles generic
	//  memory clean-up and TSAPI stream failures. SetWndPtr() is device-specific. If
	//  the app handled more than 1 device, they could have the same or different
	//  windows handling their events.
	pTsapi->SetWndPtr(this);

	UpdateData(FALSE);
	
	CDialog::OnTimer(nIDEvent);
}

// this method is called when the Call button is pressed
void CSampleDlg::OnCall() 
{
	// retrieve the data out of the dialog
	UpdateData(TRUE);

	// retrieve the device record for the monitored device
	DeviceID_t deviceID;
	lstrcpy(deviceID, m_DeviceID);
	CTsapiDevice* pTsapi = m_TsapiWndPtr->GetDeviceRecord(deviceID);
	if(!pTsapi)
	{
		AfxMessageBox("device record not found");
		PostMessage(WM_COMMAND, IDCANCEL, 0);
		return;
	}

	// make a call from monitored device to the "to device"
	// use the device record pointer as the invoke ID
	RetCode_t rc = cstaMakeCall(m_TsapiWndPtr->m_AcsHandle, (InvokeID_t)pTsapi, (DeviceID_t*)&deviceID,
									(DeviceID_t*)(LPCTSTR)m_ToDeviceID, NULL);
	if(rc < 0)
	{
		AfxMessageBox("make call failed");
		return;
	}

	// don't let anything change now
 	m_CallControl.EnableWindow(FALSE);
	m_ToDeviceIDControl.EnableWindow(FALSE);

 	return;
}

// this method is called when the Hang Up button is pressed
void CSampleDlg::OnHangup() 
{
	// retrieve the data out of the dialog
	UpdateData(TRUE);

	// retrieve the device record for the monitored device
	DeviceID_t deviceID;
	lstrcpy(deviceID, m_DeviceID);
	CTsapiDevice* pTsapi = m_TsapiWndPtr->GetDeviceRecord(deviceID);
	if(!pTsapi)
	{
		AfxMessageBox("device record not found");
		PostMessage(WM_COMMAND, IDCANCEL, 0);
		return;
	}

	// retrieve the connection ID for this call that is associated with
	//  the monitored device - this is the "local" connection
	ConnectionID_t connID;
	if(!pTsapi->GetLocalConnection(m_CallHandle, connID))
	{
		AfxMessageBox("no active call found");
		return;
	}

	// clear the connection
	// use the device record pointer as the invoke ID
	RetCode_t rc = cstaClearConnection(m_TsapiWndPtr->m_AcsHandle, (InvokeID_t)pTsapi,
											&connID, NULL);
	if(rc < 0)
	{
		AfxMessageBox("clear connection failed");
		return;
	}

	// nothing to hang-up anymore, disable that button
 	m_HangUpControl.EnableWindow(FALSE);

	return;
}

// this method enables/disables buttons (and input fields) based on current status
void CSampleDlg::DoButtonUpdates() 
{
	// enable Hang-Up button if a call is active
	m_HangUpControl.EnableWindow(m_CallHandle != 0);
	// enable Call button if no call is active AND 
	//	the "Make Call To" field is not empty 
	m_CallControl.EnableWindow((m_CallHandle == 0) && !m_ToDeviceID.IsEmpty());
	// enable the "Make Call To" field if no call is active
	m_ToDeviceIDControl.EnableWindow(m_CallHandle == 0);
}

// this method is called on every update to the "Make Call To" field.
void CSampleDlg::OnChangeTodeviceid() 
{
	// retrieve the data out of the dialog
	UpdateData(TRUE);
	// enable the Call button if the "Make Call To" field is not empty
	m_CallControl.EnableWindow(!m_ToDeviceID.IsEmpty());	
}

// this method is called when the CSTA DLL returns failure on an attempt
//  to retrieve a TSAPI message.
// the failure code is in wParam. It is a RetCode_t and is one of the
//  failure codes defined in acs.h.
LRESULT CSampleDlg::OnTsapiFailure(WPARAM wParam, LPARAM lParam)
{
	char tempStr[10];
	sprintf(tempStr, "TSAPI Message Retrieval Failure: %d", wParam);
	AfxMessageBox(tempStr);
	PostMessage(WM_COMMAND, IDCANCEL, 0);
	return 0;
}

// this method is called when an ACS Unsolicited event is received
LRESULT CSampleDlg::OnTsapiAcsUnsolicited(WPARAM wParam, LPARAM lParam)
{
	char tempStr[10];
	sprintf(tempStr, "TSAPI Stream Failure: %d",
			m_TsapiWndPtr->m_EventBufPtr->event.acsUnsolicited.u.failureEvent.error);
	AfxMessageBox(tempStr);
	PostMessage(WM_COMMAND, IDCANCEL, 0);
	return 0;
}

// this method is called when a CSTA Confirmation event is received
LRESULT CSampleDlg::OnTsapiCstaConfirmation(WPARAM wParam, LPARAM lParam)
{
	switch(m_TsapiWndPtr->m_EventBufPtr->eventHeader.eventType)
	{
	 case CSTA_MAKE_CALL_CONF:
	 case CSTA_CLEAR_CONNECTION_CONF:
		// let the unsolicited events take care of any updates
		// Generally, nothing should be done with positive confirmation
		//  events - the unsolicited events should drive the application's
		//  behaviour.
		break;
	 case CSTA_UNIVERSAL_FAILURE_CONF:
		// something more elaborate should probably be done when
		//  a request fails!
		AfxMessageBox("requested operation failed");
		break; 
	 default:
		AfxMessageBox("unexpected response to requested operation");
		break; 
	}

	return 0;
}

// this method is called when a CSTA Unsolicited event is received
LRESULT CSampleDlg::OnTsapiCstaUnsolicited(WPARAM wParam, LPARAM lParam)
{
	// Get the device record pointer out of the lParam
	// Get the unsolicited event pointer out of the event buffer (for
	//  code efficiency and typing efficiency)
	CTsapiDevice* pTsapi = (CTsapiDevice*)lParam;
	CSTAUnsolicitedEvent* pUns = &(m_TsapiWndPtr->m_EventBufPtr->event.cstaUnsolicited);

	switch(m_TsapiWndPtr->m_EventBufPtr->eventHeader.eventType)
	{
	 // this is not a call-control event. just end the app if the monitor
	 //  of the device was terminated for any reason.
	 case CSTA_MONITOR_ENDED:
		AfxMessageBox("monitor failure");
		PostMessage(WM_COMMAND, IDCANCEL, 0);
		return 0;

	 // these are call-control events...
	 case CSTA_SERVICE_INITIATED: // null -> initiated (LOCAL only)
	 {
		ConnectionID_t* pConnID = &(pUns->u.serviceInitiated.initiatedConnection);

		// this is a new call - track it
		long callHandle = pTsapi->CreateCallFromLocalConnection(*pConnID);
		if(callHandle == 0)
		{
			AfxMessageBox("no memory to add new call");
			return 0;
		}

		// remember this call and set local status
		m_CallHandle = callHandle;
		// this event is always for the local connection
		m_LocalStatus = _T("Initiated");
		break;
	 }
	 case CSTA_ORIGINATED: // null/initiated -> connected (LOCAL only)
	 {
		ConnectionID_t* pConnID = &(pUns->u.originated.originatedConnection);

		// Attempt to find the call associated with the connection ID
		//  if a SERVICE INITIATED event was received, this call will
		//   be found.
		//  if no SERVICE INITIATED event was received, then this event is the
		//   first event received.
		long callHandle = pTsapi->GetCallHandle(*pConnID, TRUE);

		if(callHandle == 0)
		{
			// assume this is a new call - track it
			callHandle = pTsapi->CreateCallFromLocalConnection(*pConnID);
			if(callHandle == 0)
			{
				AfxMessageBox("no memory to add new call");
				return 0;
			}
			m_CallHandle = callHandle;
		}

		// this event is always for the local connection
		m_LocalStatus = _T("Connected");

		// get the number dialed out of calledDevice
		// For calls originated via the phone, this is the first chance to
		//  place the information into the dialog.
		// Modifying the "To Device" here and in other events illustrates
		//  how the PBX and/or PBX Driver what has been dialled. Apps need
		//  to be aware that this can happen.
		if(pUns->u.originated.calledDevice.deviceIDStatus == ID_PROVIDED)
		{
			m_ToDeviceID = pUns->u.originated.calledDevice.deviceID;
		}
		else
		{
			m_ToDeviceID = _T("Not provided");
		}
		break;
	 }
	 case CSTA_DELIVERED: // null -> alerting
	 {
		ConnectionID_t* pConnID = &(pUns->u.delivered.connection);
 		SubjectDeviceID_t* pSubjectDevice = &(pUns->u.delivered.alertingDevice);
 		CalledDeviceID_t* pCalledDevice = &(pUns->u.delivered.calledDevice);

		// don't bother storing remote connection while it is still alerting...

		// Find the call handle based on call ID in given connection ID
		// Unfortunately, we must crack open the ConnID here (the FALSE indicates
		//  an inexact match - just match on callID). Looking at the callID is
		//  not guaranteed to work on 100% of all PBX drivers - some implementations
		//  do not guarantee unique call IDs, only unique connection IDs.
		//  There are 2 alternatives to this, neither one of which is any better
		//  than this (and may even be worse).
		//  1. Look at the calledDevice. The app should know the number that was called,
		//		and could store that number with the local connection ID. When execution
		//		reaches here, search for the called device and return the callHandle. This
		//		has 2 problems: a. calledDevice is not guaranteed to be supplied from all PBX
		//		drivers; b. if 2 calls to the same device are in progress simultaneously, which
		//		callHandle is for which device?
		//  2. Do a cstaSnapshotCall() on the connection ID passed in here. That will give you
		//		a picture of what is happening with that connID - including the connID in the
		//		call that is associated with the monitored device. The snapshot can be used to
		//		match that local connID to what is stored in the app. This delays handling of
		//		the event with asynchronous queries to the server - in the meantime other things
		//		are happening to the call!
		long callHandle = pTsapi->GetCallHandle(*pConnID, FALSE);
		if(callHandle == 0)
		{
			return 0;
		}

		// for outgoing calls, this event is always for the remote connection
		m_RemoteStatus = _T("Alerting");

		// get the called device out of subjectDevice or calledDevice
		if(pSubjectDevice->deviceIDStatus == ID_PROVIDED)
		{
			m_ToDeviceID = pSubjectDevice->deviceID;
		}
		else if(pCalledDevice->deviceIDStatus == ID_PROVIDED)
		{
			m_ToDeviceID = pCalledDevice->deviceID;
		}
		else
		{
			m_ToDeviceID = _T("Not provided");
		}
		break;
	 }
	 case CSTA_NETWORK_REACHED: // null -> connected (REMOTE only)
	 {
		ConnectionID_t* pConnID = &(pUns->u.networkReached.connection);
		SubjectDeviceID_t* pSubjectDevice = &(pUns->u.networkReached.trunkUsed);
		CalledDeviceID_t* pCalledDevice = &(pUns->u.networkReached.calledDevice);

		// Find the call handle based on call ID in given connection ID
		// Unfortunately, we must crack open the ConnID here (the FALSE indicates
		//  an inexact match - just match on callID). Looking at the callID is
		//  not guaranteed to work on 100% of all PBX drivers - some implementations
		//  do not guarantee unique call IDs, only unique connection IDs.
		//  There are 2 alternatives to this, neither one of which is any better
		//  than this (and may even be worse).
		//  1. Look at the calledDevice. The app should know the number that was called,
		//		and could store that number with the local connection ID. When execution
		//		reaches here, search for the called device and return the callHandle. This
		//		has 2 problems: a. calledDevice is not guaranteed to be supplied from all PBX
		//		drivers; b. if 2 calls to the same device are in progress simultaneously, which
		//		callHandle is for which device?
		//  2. Do a cstaSnapshotCall() on the connection ID passed in here. That will give you
		//		a picture of what is happening with that connID - including the connID in the
		//		call that is associated with the monitored device. The snapshot can be used to
		//		match that local connID to what is stored in the app. This delays handling of
		//		the event with asynchronous queries to the server - in the meantime other things
		//		are happening to the call!
		long callHandle = pTsapi->GetCallHandle(*pConnID, FALSE);
		if(callHandle == 0)
		{
			return 0;
		}

		// now add the connection to the call - while this connection is in
		//  the call, all future events should not need to use GetCallHandle() with FALSE,
		//  they should all use TRUE.

		if(!pTsapi->AddConnectionToCall(callHandle, *pConnID))
		{
			AfxMessageBox("no memory to add new connection to call");
			return 0;
		}

		// This event is always for the remote connection
 		m_RemoteStatus = _T("Connected");

		// get the trunk used out of subjectDevice or calledDevice
		if(pSubjectDevice->deviceIDStatus == ID_PROVIDED)
		{
			m_ToDeviceID = pSubjectDevice->deviceID;
		}
		else if(pCalledDevice->deviceIDStatus == ID_PROVIDED)
		{
			m_ToDeviceID = pCalledDevice->deviceID;
		}
		else
		{
			m_ToDeviceID = _T("Not provided");
		}
		break;
	 }
	 case CSTA_ESTABLISHED:	// alerting/connected -> connected
	 {
		ConnectionID_t* pConnID = &(pUns->u.established.establishedConnection);
 		SubjectDeviceID_t* pSubjectDevice = &(pUns->u.established.answeringDevice);
 		CallingDeviceID_t* pCallingDevice = &(pUns->u.established.callingDevice);
 		CalledDeviceID_t* pCalledDevice = &(pUns->u.established.calledDevice);

  		BOOL subjectProvided = (pSubjectDevice->deviceIDStatus == ID_PROVIDED);
		BOOL callingProvided = (pCallingDevice->deviceIDStatus == ID_PROVIDED);
		BOOL calledProvided = (pCalledDevice->deviceIDStatus == ID_PROVIDED);

		// Attempt to find the call associated with the connection ID
		long callHandle = pTsapi->GetCallHandle(*pConnID, TRUE);
		if(callHandle == 0)
		{
			// if the call is outgoing to an internal destination (no NETWORK REACHED event
			//  has been received) we will get here.

			BOOL outgoing = FALSE;

			// is it outgoing?
			if(subjectProvided)
			{
				outgoing = !(m_DeviceID == pSubjectDevice->deviceID);
			}
			else if(callingProvided)
			{
				outgoing = (m_DeviceID == pCallingDevice->deviceID);
			}
			else if(calledProvided)
			{
				outgoing = !(m_DeviceID == pCalledDevice->deviceID);
			}
			else
			{
				// nothing we can do if no IDs are provided!
				return 0;
			}

			// if it is NOT outgoing, ignore event - return here
			if(!outgoing)
			{
				return 0;
			}

			// Find the call handle based on call ID in given connection ID
			// Unfortunately, we must crack open the ConnID here (the FALSE indicates
			//  an inexact match - just match on callID). Looking at the callID is
			//  not guaranteed to work on 100% of all PBX drivers - some implementations
			//  do not guarantee unique call IDs, only unique connection IDs.
			//  There are 2 alternatives to this, neither one of which is any better
			//  than this (and may even be worse).
			//  1. Look at the calledDevice. The app should know the number that was called,
			//		and could store that number with the local connection ID. When execution
			//		reaches here, search for the called device and return the callHandle. This
			//		has 2 problems: a. calledDevice is not guaranteed to be supplied from all PBX
			//		drivers; b. if 2 calls to the same device are in progress simultaneously, which
			//		callHandle is for which device?
			//  2. Do a cstaSnapshotCall() on the connection ID passed in here. That will give you
			//		a picture of what is happening with that connID - including the connID in the
			//		call that is associated with the monitored device. The snapshot can be used to
			//		match that local connID to what is stored in the app. This delays handling of
			//		the event with asynchronous queries to the server - in the meantime other things
			//		are happening to the call!
			callHandle = pTsapi->GetCallHandle(*pConnID, FALSE);
			if(callHandle == 0)
			{
				return 0;
			}

			// now add the connection to the call - while this connection is in
			//  the call, all future events should not need to use GetCallHandle() with FALSE,
			//  they should all use TRUE.

			if(!pTsapi->AddConnectionToCall(callHandle, *pConnID))
			{
				AfxMessageBox("no memory to add new connection to call");
				return 0;
			}
		}

		// for outgoing calls, this event is always for the remote connection
		m_RemoteStatus = _T("Connected");

		// get the answering device out of subjectDevice or calledDevice
		if(subjectProvided)
		{
			m_ToDeviceID = pSubjectDevice->deviceID;
		}
		else if(calledProvided)
		{
			m_ToDeviceID = pCalledDevice->deviceID;
		}
		else
		{
			m_ToDeviceID = _T("Not provided");
		}
		break;
	 }
	 case CSTA_CALL_CLEARED: // * -> null (for all connections in call)
	 {
		ConnectionID_t* pConnID = &(pUns->u.callCleared.clearedCall);

		// remove the call associated with the connection ID
		long callHandle = pTsapi->RemoveCallViaConnection(*pConnID);
		if(callHandle == 0)
		{
			//  If the call was not known, just return
			return 0;
		}

		// reset information
		m_ToDeviceID.Empty();
		m_RemoteStatus = m_LocalStatus = _T("Null");
		m_CallHandle = 0;
		break;
	 }
	 case CSTA_CONNECTION_CLEARED: // * -> null
	 {
		ConnectionID_t* pConnID = &(pUns->u.connectionCleared.droppedConnection);

		// remove the call associated with the connection ID
		long callHandle = pTsapi->RemoveCallViaConnection(*pConnID);
		if(callHandle == 0)
		{
			//  If the call was not known, just return
			return 0;
		}

		// reset information
		m_ToDeviceID.Empty();
		m_RemoteStatus = m_LocalStatus = _T("Null");
		m_CallHandle = 0;
		break;
	 }
	 default:
		// unhandled message - ignore it
		return 0;
	}

	// update the status of buttons and fields
	DoButtonUpdates();
	// update the contents of fields
	UpdateData(FALSE);
	return 0;
}


